#include "data.lua"

local function round(x, n)
    n = math.pow(10, n or 0)
    x = x * n
    if x >= 0 then x = math.floor(x + 0.5) else x = math.ceil(x - 0.5) end
    return x / n
end

local function optionsSlider(val, min, max, roundd)
	UiPush()
		val = (val-min) / (max-min)
		local w = 400
		local done = false
		UiAlign("center middle")
        UiRect(w, 3)
		UiTranslate(-0.5*w, 1)
		val, done = UiSlider("ui/common/dot.png", "x", val*w, 0, w) / w
        UiTranslate(w*val, 23)
        val = round(val*(max-min)+min, roundd)
        UiText(val)
	UiPop()
	return val, done
end

--init
function init()
    DefaultData(true)
	
    --global
	global = {}
	global.debug = 			false
    global.page =           1
    global.ver =            "1.7.1"
    global.desgamever =     "1.0.0"
    global.destdmpver =     "0.2.2 (obsolete)"
    global.scroll =         0
    global.maxscroll =      0
    global.changingkb =     false

    --ui
    ui = {}
    ui.data = {
        {
            id = "about",
            title = "About", 
            image = "MOD/image/a-utils_small", 
            content = {
                {type="icon", content="MOD/image/a-utils_large"},
                {type="title", content="Artzert's Utilities"},
                {type="divider", content="About"},
                {type="desc", content="A well made and integrated set of mods that aim to enhance sandbox and campaign gameplay!"},
                {type="divider", content="Navigation"},
                {type="desc", content="To change a certain mods' settings, click it on in the list on the right of this window."},
                {type="desc", content="You will see an array of setting modules and info."},
                {type="desc", content="Below are examples of these modules and how to use them."},
                {type="divline"},
                {type="subtitle", content="Keybinds"},
                {type="desc", content="Click the keybind to change it, while changing keybinds you cannot do anything else."},
                {type="keybind", content="savegame.mod.example.keybind", label="Example Keybind: "},
                {type="divline"},
                {type="subtitle", content="Toggles"},
                {type="desc", content="Click the button to toggle it, these can only be true or false"},
                {type="toggle", content="savegame.mod.example.toggle", label="Example Toggle: "},
                {type="divline"},
                {type="subtitle", content="Sliders"},
                {type="desc", content="Drag the ball to change the value. The range and step can differ depending on the setting"},
                {type="slider", content="savegame.mod.example.slider", mod=0.01, round=2, range={0,1} },
                {type="divline"},
                {type="subtitle", content="Scrolling"},
                {type="desc", content="Sometimes, the page will be too long to show all at once. So you will have to scroll down."},
                {type="desc", content="Try scrolling down and see what you can find!"},
                {type="whitespace", content=1000},
                {type="desc", content="Hooray, you have learned how to use this options menu!"},
            }
        },
        {id = "whitespace"},
        {
            id = "general",
            title = "General", 
            image = "MOD/image/general_small", 
            content = {
                {type="icon", content="MOD/image/general_large"},
                {type="title", content="General"},
                {type="divider", content="Toggle Mods"},
                {type="desc", content="Choose which mods to activate."},
                {type="toggle", content="savegame.mod.global.fly.enabled", label="Flight: "},
                {type="toggle", content="savegame.mod.global.sprint.enabled", label="Sprint: "},
                {type="toggle", content="savegame.mod.global.phys.enabled", label="Pause Physics: "},
                {type="toggle", content="savegame.mod.global.hands.enabled", label="Hands: "},
                {type="toggle", content="savegame.mod.global.horn.enabled", label="Horn: "},
                {type="toggle", content="savegame.mod.global.debris.enabled", label="Debris Remover: "},
                {type="toggle", content="savegame.mod.global.speedmtr.enabled", label="Speedometer: "},
                {type="toggle", content="savegame.mod.global.tctrl.enabled", label="Time control: "},
                {type="toggle", content="savegame.mod.global.imoh.enabled", label="I'm Outta Here: "},
                {type="toggle", content="savegame.mod.global.calarm.enabled", label="Car Alarm: "},
                {type="toggle", content="savegame.mod.global.advengine.enabled", label="Car Locking: "},
                {type="divider", content="TDMP"},
                {type="desc", content="The host of a session controls which mods are activated."},
                {type="desc", content="To reduce desync, some settings are overridden."},
                {type="toggle", content="savegame.mod.global.tdmpoverides", label="Use overrides: "},
                {type="desc", content="Will print stuff to console with info about tdmp communication."},
                {type="toggle", content="savegame.mod.global.tdmpdebug", label="TDMP Debug: "},
                {type="divider", content="Miscellaneous"},
                {type="desc", content="Should the sidebar be on by default?"},
                {type="toggle", content="savegame.mod.global.sideinfo", label="Display Sidebar Default: "},
                {type="keybind", content="savegame.mod.global.sbkb", label="Toggle Sidebar: "},
                {type="subtitle", content="Reset Settings"},
                {type="desc", content="Reset all settings to default"},
                {type="resetbutton"},
            }
        },
        {
            id = "fly",
            title = "Flight", 
            image = "MOD/image/flight_small", 
            content = {
                {type="icon", content="MOD/image/flight_large"},
                {type="title", content="Flight"},
                {type="divider", content="Info"},
                {type="desc", content="Allows for flight, noclip and freecam. Now optimized to be still (player will not slowly rise / fall)."},
                {type="divider", content="Keybinds"},
                {type="keybind", content="savegame.mod.fly.DownKey", label="Move Down: "},
                {type="keybind", content="savegame.mod.fly.SpeedKey", label="Speed Boost: "},
                {type="keybind", content="savegame.mod.fly.PreciseKey", label="Precision (Slow): "},
                {type="keybind", content="savegame.mod.fly.NoclipKey", label="Noclip: "},
                {type="keybind", content="savegame.mod.fly.FreecamKey", label="Freecam: "},
                {type="desc", content="You can choose if the mod checks for a double tap or just a single press. (to toggle fly)"},
                {type="toggle", content="savegame.mod.fly.doDoubleTap", label="Require Double Tap: "},
                {type="keybind", content="savegame.mod.fly.flykey", label="Fly: "},
                {type="desc", content="Freecam returns to the player, noclip teleports the player to the exit position."},
                {type="divider", content="Toggles"},
                {type="desc", content="You can restrict fly to sandbox maps only."},
                {type="toggle", content="savegame.mod.fly.sandboxOnly", label="Sandbox only: "},
                {type="desc", content="Heal the player when descending so you dont take damage when hitting the ground."},
                {type="toggle", content="savegame.mod.fly.healWhenDown", label="Heal On Descend: "},
                {type="desc", content="Cancel flight when hitting the ground."},
                {type="toggle", content="savegame.mod.fly.cog", label="Cancel On Ground: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Fly Timing"},
                {type="desc", content="Maximum time between two space presses, higher values make entering flight easier."},
                {type="slider", content="savegame.mod.fly.FlyTiming", mod=1, round=0, range={10,25} },
                {type="subtitle", content="Base Speeds"},
                {type="desc", content="Base speed for regular flight:"},
                {type="slider", content="savegame.mod.fly.flyBaseSpeed", mod=1, round=0, range={5,25} },
                {type="desc", content="Base speed for noclip and freecam:"},
                {type="slider", content="savegame.mod.fly.camBaseSpeed", mod=1, round=0, range={15,40} },
                {type="subtitle", content="Speed Modifiers"},
                {type="desc", content="Speed multiplied when holding down the boost key:"},
                {type="slider", content="savegame.mod.fly.flyingMod", mod=0.01, round=2, range={1.5,3} },
                {type="desc", content="Speed multiplied when holding down the precision key."},
                {type="slider", content="savegame.mod.fly.preciseMod", mod=0.01, round=2, range={0.25,0.8} },
            }
        },
        {
            id = "sprint",
            title = "Sprint & Walk", 
            image = "MOD/image/sprint_small", 
            content = {
                {type="icon", content="MOD/image/sprint_large"},
                {type="title", content="Sprint & Walk"},
                {type="divider", content="Info"},
                {type="desc", content="Allows you to speed up / slow down. You can limit the sprint amount (or make it infinite)."},
                {type="divider", content="Keybinds"},
                {type="keybind", content="savegame.mod.sprint.sprintkey", label="Sprint: "},
                {type="keybind", content="savegame.mod.sprint.walkKey", label="Walk: "},
                {type="divider", content="Toggles"},
                {type="desc", content="You can disable bluetide, since sprint serves the same purpose."},
                {type="toggle", content="savegame.mod.sprint.disableBT", label="Disable BlueTide: "},
                {type="desc", content="You can choose if you want sprit to be limited (=you can run out and have to regen the sprint meter)."},
                {type="toggle", content="savegame.mod.sprint.limit", label="Limit Sprint: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Sprint limit"},
                {type="desc", content="Limit (in seconds) of sprint you have."},
                {type="slider", content="savegame.mod.sprint.time", mod=1, round=0, range={5,15} },
                {type="subtitle", content="Speeds"},
                {type="desc", content="Sprint speed:"},
                {type="slider", content="savegame.mod.sprint.speed", mod=1, round=0, range={5,15} },
                {type="desc", content="Walk speed:"},
                {type="slider", content="savegame.mod.sprint.walkspeed", mod=0.01, round=2, range={1,4} },
            }
        },
        {
            id = "phys",
            title = "Pause Physics", 
            image = "MOD/image/pausephysics_small", 
            content = {
                {type="icon", content="MOD/image/pausephysics_large"},
                {type="title", content="Pause Physics"},
                {type="divider", content="Info"},
                {type="desc", content="Allows you to pause physics."},
                {type="desc", content="Paused objects might move the player / have elongated particles."},
                {type="divider", content="Keybinds"},
                {type="keybind", content="savegame.mod.physics.toggleKB", label="Pause/Resume: "},
                {type="divider", content="Toggles"},
                {type="desc", content="Should this mod forget the movement of objects after they were frozen?"},
                {type="toggle", content="savegame.mod.physics.cancelMvmnt", label="Cancel Movement: "},
                {type="desc", content="Smart update tries to eliminate the lag after re-enabling physics by only updating bodies that were active before the freeze."},
                {type="toggle", content="savegame.mod.physics.smartUpdate", label="Smart Update: "},
                {type="desc", content="Should this mod enforce frozen objects (keep freezing objects while physics are off)."},
                {type="toggle", content="savegame.mod.physics.enforce", label="Enforce Freeze: "},
            }
        },
        {
            id = "hands",
            title = "Hands", 
            image = "MOD/image/hands_small", 
            content = {
                {type="icon", content="MOD/image/hands_large"},
                {type="title", content="Hands"},
                {type="divider", content="Info"},
                {type="desc", content="Adds a hands tool with no model so you can explore without any model obscuring your sight."},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Hands Toolgroup"},
                {type="desc", content="Toolgroup of the hands tool."},
                {type="slider", content="savegame.mod.hands.toolgroup", mod=1, round=0, range={1,6} },
            }
        },
        {
            id = "horn",
            title = "Horn", 
            image = "MOD/image/horn_small", 
            content = {
                {type="icon", content="MOD/image/horn_large"},
                {type="title", content="Horn"},
                {type="divider", content="Info"},
                {type="desc", content="Adds a horn to vehicles based on their type (water / land) and size (small/medium/large)"},
                {type="desc", content="Might not work with other vehicle frameworks and non-vanilla vehicles."},
                {type="divider", content="Keybinds"},
                {type="keybind", content="savegame.mod.horn.hornkey", label="Horn: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Horn Speed"},
                {type="desc", content="Fade in and out speed of the horn (higher = faster)."},
                {type="slider", content="savegame.mod.horn.fadespeed", mod=1, round=0, range={1,10} },
            }
        },
        {
            id = "debris",
            title = "Debris Remover", 
            image = "MOD/image/debris_small", 
            content = {
                {type="icon", content="MOD/image/debris_large"},
                {type="title", content="Debris Remover"},
                {type="divider", content="Info"},
                {type="desc", content="Removes any bodies below / above limit values. (ignores the player's tool and bodies tagged with tool, valuable or interact)"},
                {type="desc", content="Might not work with plugins / maps that hide objects by putting them in limbo below the map."},
                {type="desc", content="If you're a map/mod author you could add some of the aforementioned tags to fix this issue."},
                {type="divider", content="Toggles"},
                {type="desc", content="Clear debris below -19."},
                {type="toggle", content="savegame.mod.debris.bottom", label="Bottom: "},
                {type="desc", content="Clear debris above 100000."},
                {type="toggle", content="savegame.mod.debris.top", label="Top: "},
                {type="desc", content="Slow down scanning speed to save fps."},
                {type="toggle", content="savegame.mod.debris.lightmode", label="Light mode: "},
            }
        },
        {
            id = "speedmtr",
            title = "Speedometer", 
            image = "MOD/image/speedmtr_small", 
            content = {
                {type="icon", content="MOD/image/speedmtr_large"},
                {type="title", content="Speedometer"},
                {type="divider", content="Info"},
                {type="desc", content="Displays a small bar and number with the vehicle speed near the vehicle condition bar."},
                {type="divider", content="Toggles"},
                {type="desc", content="Disable to use mph instead of km/h."},
                {type="toggle", content="savegame.mod.speedmtr.kph", label="Use Km/h: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Speed Bar Limit"},
                {type="desc", content="The speed (always in km/h!) at which the bar maxes out."},
                {type="slider", content="savegame.mod.speedmtr.lim", mod=1, round=0, range={100,250} },
            }
        },
        {
            id = "tctrl",
            title = "Time Control", 
            image = "MOD/image/clock_small", 
            content = {
                {type="icon", content="MOD/image/clock_large"},
                {type="title", content="Time Control"},
                {type="divider", content="Info"},
                {type="desc", content="Allows you to slow down time."},
                {type="desc", content="Very slow values might cause funky collisions."},
                {type="divider", content="Keybinds"},
                {type="keybind", content="savegame.mod.tctrl.up", label="Speed Up: "},
                {type="keybind", content="savegame.mod.tctrl.down", label="Slow Dowm: "},
                {type="keybind", content="savegame.mod.tctrl.toggle", label="Toggle manipulation: "},
                {type="divider", content="Toggles"},
                {type="desc", content="Should this mode ease in and out of realtime?"},
                {type="toggle", content="savegame.mod.tctrl.fade", label="Ease Time: "},
                {type="desc", content="Should flight and camera speeds be adjusted so they don't slow down?"},
                {type="toggle", content="savegame.mod.tctrl.immune", label="Adjust Flight: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Adjustment Percision"},
                {type="desc", content="The amount by which you increase / decrease time."},
                {type="slider", content="savegame.mod.tctrl.incrmnt", mod=0.01, round=2, range={0.01,0.05} },
            }
        },
        {
            id = "imoh",
            title = "I'm Outta Here", 
            image = "MOD/image/rocket_small", 
            content = {
                {type="icon", content="MOD/image/rocket_large"},
                {type="title", content="I'm Outta Here"},
                {type="divider", content="Info"},
                {type="desc", content="Accelerates the player / vehicle by multiplying velocity."},
                {type="desc", content="Setting the multiplier too high might create some bugs."},
                {type="divider", content="Keybinds"},
                {type="keybind", content="savegame.mod.imoh.key", label="Launch: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Launch multiplier"},
                {type="desc", content="Velocty will be multiplied by this value."},
                {type="slider", content="savegame.mod.imoh.vel", mod=1, round=0, range={2,100} },
            }
        },
        {
            id = "calarm",
            title = "Car Alarm", 
            image = "MOD/image/car_small", 
            content = {
                {type="icon", content="MOD/image/car_large"},
                {type="title", content="Car Alarm"},
                {type="divider", content="Info"},
                {type="desc", content="Locked vehicles (= engine off) will honk when damaged."},
                {type="desc", content="Entering or unlocking (see car locking mod) these vehicles stops the alarm."},
                {type="divider", content="Toggles"},
                {type="desc", content="This mod can make the lights turn on and off by turning on and off the engine."},
                {type="desc", content="This has the downside of creating a lot of noise, so you might not want to enable it."},
                {type="toggle", content="savegame.mod.calarm.doEngineCycle", label="Use Engine Method: "},
                {type="desc", content="Should the alarm activate when a car goes underwater?"},
                {type="desc", content="If false cars will also stop honking when underwater."},
                {type="toggle", content="savegame.mod.calarm.wateralarm", label="Water Alarm: "},
                {type="divider", content="Sliders"},
                {type="subtitle", content="Alarm Health Limit"},
                {type="desc", content="When is a car too damaged to honk. (0.8 -> below 80% hp no alarms)"},
                {type="slider", content="savegame.mod.calarm.limit", mod=0.01, round=2, range={0,0.8} },
            }
        },
        {
            id = "advengine",
            title = "Car Locking", 
            image = "MOD/image/engine_small", 
            content = {
                {type="icon", content="MOD/image/engine_large"},
                {type="title", content="Car Locking"},
                {type="divider", content="Info"},
                {type="desc", content="Adds car keys and the ability to lock vehicles."},
                {type="desc", content="Unlocked vehicles will keep the engine running."},
                {type="divider", content="Toggles"},
                {type="desc", content="Should cars automatically lock (when you exit them)?"},
                {type="toggle", content="savegame.mod.advengine.startreqkey", label="Auto Lock: "},
                {type="desc", content="Should unlocked cars apply the handbrake when the player exits them?"},
                {type="desc", content="Only applies when autolock is off."},
                {type="toggle", content="savegame.mod.advengine.doautopark", label="Exit Handbrake: "},
            }
        },
        {id = "whitespace"},
        {
            id = "patchnotes",
            title = "Patchnotes", 
            image = "MOD/image/info_small", 
            content = {
                {type="divider", content="Version 1.7.1"},
                {type="desc", content="Added water alarm option"},
                {type="desc", content="You can choose if car alarm restarts the engines."},
                {type="desc", content="You can choose when a car is too damaged to sound an alarm."},
                {type="divider", content="Version 1.7.0"},
                {type="desc", content="Added car alarm mod. (hooray!)"},
                {type="desc", content="Added car locking mod."},
                {type="desc", content="TDMP is no longer supported by this mod since it has been cancelled."},
                {type="divider", content="Version 1.6.1"},
                {type="desc", content="Removed some debug print stuff."},
                {type="desc", content="Added light mode to debris cleaner."},
                {type="desc", content="With everything enabled this mod drops fps by about 7% now (using light mode for debris)."},
                {type="divider", content="Version 1.6"},
                {type="desc", content="Fixed cancel on ground."},
                {type="desc", content="Added horn to spawned vehicles (experimental version)."},
                {type="desc", content="Double tapping the fly key now exits noclip and freecam."},
                {type="divider", content="Version 1.5"},
                {type="desc", content="Added TDMP config sync."},
                {type="desc", content="Added TDMP pause physics sync."},
                {type="desc", content="Added TDMP time control sync."},
                {type="desc", content="Added TDMP horn sync."},
                {type="desc", content="Fixed IMOH while sprinting."},
                {type="divider", content="Version 1.4"},
                {type="desc", content="Improved debris cleaner tag system."},
                {type="desc", content="Added option to remove debris above 100000."},
                {type="desc", content="Added keybind to toggle the sidebar."},
                {type="desc", content="Added I'm Outta Here mod."},
                {type="desc", content="Updated tdmp api."},
                {type="divider", content="Version 1.3-Hotfix"},
                {type="desc", content="Debris remover no longer clears bodies tagged with section."},
                {type="divider", content="Version 1.3"},
                {type="desc", content="Added time control mod."},
                {type="desc", content="Removed bullet time."},
                {type="desc", content="Noclip no longer glitches out when moving into border."},
                {type="desc", content="Updated thumbnail."},
                {type="desc", content="Added the beginnings of TDMP compatibility. (some mods should already work)"},
                {type="desc", content="Made it more obvious when mods are disabled."},
                {type="desc", content="Added cancel on ground option for flight."},
                {type="divider", content="Version 1.2"},
                {type="desc", content="Added speedometer mod."},
                {type="desc", content="Added more options for fly keybinds."},
                {type="desc", content="Mod now remembers vehicle types, so destruction has no effect on the horn type."},
                {type="desc", content="Added a better data system."},
                {type="divider", content="Version 1.1"},
                {type="desc", content="You can now use tools in noclip."},
                {type="desc", content="Added exit button & patchnotes."},
                {type="desc", content="Added spacing in the options list."},
                {type="desc", content="Fixed smart update (pause physics)."},
            }
        },
        {
            id = "bugs",
            title = "Found a bug?", 
            image = "MOD/image/bug_small", 
            content = {
                {type="icon", content="MOD/image/bug_large"},
                {type="title", content="Bug Reporting"},
                {type="desc", content="Use the steam comments / topics of this mod to report bugs / request features."},
                {type="desc", content="Always double check if the cause of the bug is actually my mod and not something else!"},
            }
        }
    }


    if global.debug then
        DefaultData(false)
    end

end

--draw
function draw()
	
    --draw sidebars
    UiPush()
        UiColor(1, 1, 1)
        UiAlign("center top")
        UiTranslate(UiCenter()-500,0)
        UiRect(2,UiHeight())
        UiTranslate(1000,0)
        UiRect(2,UiHeight())
        ---draw scrollbar
        if global.maxscroll > 0 then
            UiTranslate(0,(global.scroll/global.maxscroll)*(UiHeight()-50))
            UiRect(6,50)
        end
    UiPop()

    --draw version number
    UiPush()
        UiColor(1, 1, 1)
        UiAlign("left middle")
        UiTranslate(UiCenter()+500,UiHeight()-10)
        UiFont("regular.ttf", 15)
        UiText("Game: " .. GetVersion())
        UiTranslate(0,-18)
        UiText("Data Ver: " .. GetInt("savegame.mod.general.version"))
        UiTranslate(0,-18)
        UiText("Mod: " .. global.ver .. " ("..global.desgamever..")")
    UiPop()


    --draw selector on left of sidebar
    UiPush()
        UiColor(1, 1, 1)
        UiAlign("right middle")
        UiTranslate(UiCenter()-510,45)
        UiFont("bold.ttf", 25)
        for i,v in pairs(ui.data) do
            if v.id == "whitespace" then
                UiTranslate(0,35)
            else
                local title = v.title
                local enabled = ((GetBool("savegame.mod.global.".. v.id ..".enabled")) or (i<=3) or (i>=#ui.data-2))
                if UiIsMouseInRect(UiCenter()-500, 35) then
                    UiTranslate(10,0)
                    UiColor(1, 1, 1, 0.3)
                    UiRect(UiCenter()-500,35)
                    UiColor(1, 1, 1, 1)
                    UiTranslate(-10,0)
                    if InputPressed("lmb") and (global.changingkb == false) then
                        global.page = i
                        global.scroll = 0
                    end
                    if not enabled then
                        title = "Mod Disabled!"
                    end
                elseif i == global.page then
                    UiTranslate(10,0)
                    UiColor(1, 1, 1, 0.15)
                    UiRect(UiCenter()-500,35)
                    UiColor(1, 1, 1, 1)
                    UiTranslate(-10,0)
                end
                if not enabled then
                    UiColor(1, 1, 1, 0.5)
                end
                UiImage(v.image..".png")
                UiTranslate(-30,0)
                UiText(title)
                UiColor(1, 1, 1, 1)
                UiTranslate(30,35)
            end
        end
    UiPop()

    --draw exit button
    UiPush()
        UiColor(1, 1, 1)
        UiAlign("right middle")
        UiTranslate(UiCenter()-510,UiHeight()-50)
        UiFont("bold.ttf", 25)
        if UiIsMouseInRect(UiCenter()-500, 35) then
            UiTranslate(10,0)
            UiColor(1, 1, 1, 0.3)
            UiRect(UiCenter()-500,35)
            UiColor(1, 1, 1, 1)
            UiTranslate(-10,0)
            if InputPressed("lmb") and (global.changingkb == false) then
                Menu()
            end
        end
        UiImage("MOD/image/exit_small.png")
        UiTranslate(-30,0)
        UiText("Exit Options")
        UiTranslate(30,35)
    UiPop()


    --draw content of current page
    UiPush()
        UiColor(1, 1, 1)
        UiAlign("center top")
        UiTranslate(UiCenter(),45-global.scroll)
        local totalsize = 0
        for i,v in pairs(ui.data[global.page]["content"]) do
            if v.type == "icon" then
                UiImage(v.content..".png")
                UiTranslate(0,105)
                totalsize = totalsize + 105
            elseif v.type == "title" then
                UiFont("bold.ttf", 50)
                UiText(v.content)
                UiTranslate(0,60)
                totalsize = totalsize + 60
            elseif v.type == "desc" then
                UiFont("regular.ttf", 20)
                UiText(v.content)
                UiTranslate(0,30)
                totalsize = totalsize + 30
            elseif v.type == "keybind" then
                UiFont("regular.ttf", 25)
                UiPush()
                    UiColor(0.2, 0.6, 1)
                    UiImageBox("ui/common/box-solid-6.png", 120, 40, 6, 6)
                UiPop()
                UiAlign("center middle")
                UiTranslate(0,20)
                local tempName
                if (global.changingkb == v.content) then tempName = "press a key" else tempName = GetString(v.content) end
                if UiTextButton(tempName, 120, 40) then
                    global.changingkb = v.content
                end
                UiPush()
                    UiTranslate(-60,0)
                    UiAlign("right middle")
                    UiText(v.label)
                UiPop()
                UiAlign("center top")
                UiTranslate(0,40)
                totalsize = totalsize + 60
            elseif v.type == "toggle" then
                UiFont("regular.ttf", 25)
                local tempName
                UiPush()
                    if GetBool(v.content) then
                        UiColor(0, 0.75, 0)
                        tempName = "true"
                    else
                        UiColor(1, 0, 0)
                        tempName = "false"
                    end
                    UiImageBox("ui/common/box-solid-6.png", 120, 40, 6, 6)
                UiPop()
                UiAlign("center middle")
                UiTranslate(0,20)
                if UiTextButton(tempName, 120, 40) then
                    SetBool(v.content, not GetBool(v.content))
                end
                UiPush()
                    UiTranslate(-60,0)
                    UiAlign("right middle")
                    UiText(v.label)
                UiPop()
                UiAlign("center top")
                UiTranslate(0,40)
                totalsize = totalsize + 60
            elseif v.type == "slider" then
                SetInt(v.content, (1/v.mod) * optionsSlider(GetInt(v.content)*v.mod, v.range[1], v.range[2], v.round))
                UiTranslate(0, 55)
                totalsize = totalsize + 55
            elseif v.type == "subtitle" then
                UiFont("bold.ttf", 25)
                UiText(v.content)
                UiTranslate(0,35)
                totalsize = totalsize + 35
            elseif v.type == "divider" then
                UiRect(1000,2)
                UiTranslate(0,5)
                UiFont("bold.ttf", 35)
                UiText(v.content)
                UiTranslate(0,40)
                UiRect(1000,2)
                UiTranslate(0,10)
                totalsize = totalsize + 55
            elseif v.type == "whitespace" then
                UiTranslate(0,v.content)
                totalsize = totalsize + v.content
            elseif v.type == "divline" then
                UiRect(1000,2)
                UiTranslate(0,5)
                totalsize = totalsize + 5
            elseif v.type == "resetbutton" then
                UiFont("regular.ttf", 20)
                UiPush()
                    UiColor(1, 0, 0)
                    UiImageBox("ui/common/box-solid-6.png", 120, 40, 6, 6)
                UiPop()
                UiAlign("center middle")
                UiTranslate(0,20)
                if UiTextButton("Reset Settings", 120, 40) then
                    DefaultData(false)
                end
                UiAlign("center top")
                UiTranslate(0,40)
                totalsize = totalsize + 60
            else
                UiFont("regular.ttf", 15)
                UiText("invalid div class!")
                UiTranslate(0,25)
                totalsize = totalsize + 25
            end
        end
        if totalsize > UiHeight() then
            global.maxscroll = totalsize - UiHeight() + 45
        else
            global.maxscroll = 0
        end
    UiPop()

    --check scroll and apply
    if global.maxscroll > 0 and (global.changingkb == false) then
        global.scroll = global.scroll - InputValue("mousewheel")*30
        if global.scroll < 0 then
            global.scroll = 0
        elseif global.scroll>global.maxscroll then
            global.scroll = global.maxscroll
        end
    end

    --check key press for keybind assignment
    local lpk = InputLastPressedKey()
    if global.changingkb ~= false and lpk ~= "" then
        SetString(global.changingkb, lpk)
        global.changingkb = false
    end
end